package com.chilunyc.fumao.api.company;

import com.chilunyc.fumao.common.exception.SystemException;
import com.chilunyc.fumao.common.request.IdsRequest;
import com.chilunyc.fumao.common.response.JsonResponse;
import com.chilunyc.fumao.common.shiro.AuthRoles;
import com.chilunyc.fumao.company.model.Company;
import com.chilunyc.fumao.company.model.CompanyAdmin;
import com.chilunyc.fumao.company.request.companyAdmin.CompanyAdminEditRequest;
import com.chilunyc.fumao.company.request.companyAdmin.CompanyAdminPageRequest;
import com.chilunyc.fumao.company.service.CompanyAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 *
 */
@Log4j
@Api(description = "集团管理员")
@RestController
@RequestMapping("/company/admin")
@PreAuthorize(AuthRoles.CompanyAdminExp)
class CompanyAdminController {

    @Autowired
    CompanyAdminService companyAdminService;


    @ApiOperation(value = "分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "request", required = true, paramType = "body", dataType = "PageRequest")
    })
    @RequestMapping(value = "/find", method = RequestMethod.POST)
    JsonResponse<Page<CompanyAdmin>> page(CompanyAdmin companyAdmin, @RequestBody CompanyAdminPageRequest request) {
        request.setCompanyId(companyAdmin.getCompanyId());
        Page<CompanyAdmin> page = companyAdminService.page(request);
        return JsonResponse.success(page);
    }

    @ApiOperation(value = "详情")
    @ApiImplicitParams({
    })
    @RequestMapping(value = "/find/{id}", method = RequestMethod.GET)
    JsonResponse<CompanyAdmin> info(CompanyAdmin companyAdmin, @PathVariable Long id) {
        CompanyAdmin companyAdminResponse = companyAdminService.findByCompanyId(id, companyAdmin.getCompanyId());
        return JsonResponse.success(companyAdminResponse);
    }


    @ApiOperation(value = "新增 | 编辑")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "request", required = true, paramType = "body", dataType = "CompanyAdmin")
    })
    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    JsonResponse edit(CompanyAdmin companyAdmin, @RequestBody @Valid CompanyAdminEditRequest request) {
        request.setCompanyId(companyAdmin.getCompanyId());
        companyAdminService.edit(request);
        return JsonResponse.success();
    }

    @ApiOperation(value = "删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "request", required = true, paramType = "body", dataType = "IdRequest")
    })
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    JsonResponse remove(CompanyAdmin companyAdmin, @RequestBody @Valid IdsRequest<Long> request) {
        if (request.getIds().contains(companyAdmin.getId())){
            throw new SystemException("100001");
        }
        companyAdminService.removeByCompanyId(request, companyAdmin.getCompanyId());
        return JsonResponse.success();
    }

    @ApiOperation(value = "重置密码")
    @ApiImplicitParams({
    })
    @RequestMapping(value = "/reset/password/{id}", method = RequestMethod.GET)
    JsonResponse resetPassword(CompanyAdmin companyAdmin, @PathVariable Long id) {
        companyAdminService.resetPassword(id, companyAdmin.getCompanyId());
        log.info(String.format("集团管理员[ %s ]重置密码 => [ %s ]", companyAdmin.getId(), id));
        return JsonResponse.success();
    }


    @ApiOperation(value = "所有")
    @ApiImplicitParams({
    })
    @RequestMapping(value = "/find/all", method = RequestMethod.GET)
    JsonResponse<List<CompanyAdmin>> all(CompanyAdminPageRequest request) {
        List<CompanyAdmin> result = companyAdminService.all(request);
        return JsonResponse.success(result);
    }

}