package com.chilunyc.fumao.api.system;

import com.chilunyc.fumao.common.request.IdsRequest;
import com.chilunyc.fumao.common.response.JsonResponse;
import com.chilunyc.fumao.common.shiro.AuthRoles;
import com.chilunyc.fumao.company.model.Company;
import com.chilunyc.fumao.company.request.company.CompanyEditRequest;
import com.chilunyc.fumao.company.request.company.CompanyPageRequest;
import com.chilunyc.fumao.company.service.CompanyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 *
 */
@Log4j
@Api(description = "集团")
@RestController
@RequestMapping("/system/company")
@PreAuthorize(AuthRoles.SystemAdminExp)
class CompanyController {

    @Autowired
    CompanyService companyService;


    @ApiOperation(value = "分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "request", required = true, paramType = "body", dataType = "PageRequest")
    })
    @RequestMapping(value = "/find", method = RequestMethod.POST)
    JsonResponse<Page<Company>> page(@RequestBody CompanyPageRequest request) {
        Page<Company> page = companyService.page(request);
        return JsonResponse.success(page);
    }

    @ApiOperation(value = "详情")
    @ApiImplicitParams({
    })
    @RequestMapping(value = "/find/{id}", method = RequestMethod.GET)
    JsonResponse<Company> info(@PathVariable Long id) {
        Company companyResponse = companyService.find(id);
        return JsonResponse.success(companyResponse);
    }


    @ApiOperation(value = "新增 | 编辑")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "request", required = true, paramType = "body", dataType = "Company")
    })
    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    JsonResponse edit(@RequestBody @Valid CompanyEditRequest request) {
        Company edit = companyService.edit(request);
        return JsonResponse.success(edit);
    }

    @ApiOperation(value = "删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "request", required = true, paramType = "body", dataType = "IdRequest")
    })
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    JsonResponse remove(@RequestBody @Valid IdsRequest<Long> request) {
        companyService.remove(request);
        return JsonResponse.success();
    }

    @ApiOperation(value = "所有")
    @ApiImplicitParams({
    })
    @RequestMapping(value = "/find/all", method = RequestMethod.GET)
    JsonResponse<List<Company>> all(CompanyPageRequest request) {
        List<Company> result = companyService.all(request);
        return JsonResponse.success(result);
    }

}