package com.chilunyc.fumao.config.swagger;

import com.chilunyc.fumao.common.response.JsonResponse;
import com.fasterxml.classmate.TypeResolver;
import com.google.common.base.Predicate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.ResponseEntity;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.schema.AlternateTypeRules;
import springfox.documentation.schema.WildcardType;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

/**
 *
 */
@Configuration
@ConditionalOnProperty(value = "app.test", havingValue = "true")
@EnableSwagger2
public class SwaggerConfig {

    @Autowired
    private TypeResolver typeResolver;

    @Bean
    Docket all() {
        return build("所有", "/", "所有接口");
    }

    /**
     * @param group
     * @param path
     * @param title
     * @return
     */
    Docket build(String group, String path, String title) {
        return new Docket(DocumentationType.SWAGGER_2)
                .select()
                .paths(includePath(path))
                .build()
                .groupName(group)
                .ignoredParameterTypes(
                )
                .apiInfo(
                        new ApiInfoBuilder()
                                .title(title)
                                .version("1.0")
                                .build())
                .genericModelSubstitutes(ResponseEntity.class)
                .alternateTypeRules(
                        AlternateTypeRules.newRule(
                                typeResolver.resolve(ResponseEntity.class, typeResolver.resolve(JsonResponse.class, WildcardType.class)),
                                typeResolver.resolve(WildcardType.class)
                        )
                )
                .useDefaultResponseMessages(false)
                ;
    }

    /**
     * @param path
     * @return
     */
    Predicate<String> includePath(final String path) {
        return new Predicate<String>() {
            @Override
            public boolean apply(String input) {
                return input.startsWith(path);
            }
        };
    }
}
