package com.chilunyc.fumao.common.util.excel;

import com.chilunyc.fumao.common.util.excel.column.ExcelColumn;
import com.chilunyc.fumao.common.util.excel.row.ExcelRow;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.springframework.util.ReflectionUtils;

import java.lang.reflect.Field;
import java.util.Map;
import java.util.TreeMap;

/**
 * 80347471@qq.com
 */
public class ExcelEntityDescription<T> {

    private Class clazz;

    private ExcelRow excelRow;

    private final Map<Integer, ExcelEntityFieldDescription> indexColumnMap = new TreeMap<>();

    public ExcelEntityDescription(Class<T> clazz) {
        this.clazz = clazz;
        this.analysis();
    }

    public <T> T read(Row row, int rowIndex) throws Exception {
        if (rowIndex == 0) {
            return null;
        }
        Object o = this.clazz.newInstance();
        short lastCellNum = row.getLastCellNum();
        for (int cellIndex = 0; cellIndex < lastCellNum; cellIndex++) {
            Cell cell = row.getCell(cellIndex);
            ExcelEntityFieldDescription fieldDescription = indexColumnMap.get(cellIndex);
            if (fieldDescription != null) {
                fieldDescription.getReader().read(o, fieldDescription, row, rowIndex, cell, cellIndex);
            }
        }
        return (T) o;
    }

    public void writeHead(XSSFSheet sheet) {
        XSSFRow row = sheet.createRow(0);
        for (Map.Entry<Integer, ExcelEntityFieldDescription> entry : indexColumnMap.entrySet()) {
            XSSFCell cell = row.createCell(entry.getKey());
            cell.setCellValue(entry.getValue().getExcelColumn().title());
        }
    }

    public void writeEntity(XSSFSheet sheet, T o, int dataIndex) throws Exception {
        XSSFRow row = sheet.createRow(dataIndex + 1);
        for (Map.Entry<Integer, ExcelEntityFieldDescription> entry : indexColumnMap.entrySet()) {
            XSSFCell cell = row.createCell(entry.getKey());
            ExcelEntityFieldDescription value = entry.getValue();
            value.getWriter().write(o, value, row, dataIndex + 1, cell, entry.getKey(), sheet);
        }
    }

    public void writeFoot(XSSFSheet sheet) {
        for (Map.Entry<Integer, ExcelEntityFieldDescription> entry : indexColumnMap.entrySet()) {
            sheet.autoSizeColumn(entry.getKey());
        }
    }

    /**
     *
     */
    private void analysis() {
        // 解析类型
        ExcelRow annotation = (ExcelRow) this.clazz.getAnnotation(ExcelRow.class);
        if (annotation != null) {
            this.excelRow = annotation;
        }

        // 解析字段
        ReflectionUtils.doWithFields(clazz, new ReflectionUtils.FieldCallback() {
            @Override
            public void doWith(Field field) throws IllegalArgumentException, IllegalAccessException {
                ExcelColumn annotation = field.getAnnotation(ExcelColumn.class);
                if (annotation != null) {
                    try {
                        field.setAccessible(true);
                        ExcelEntityFieldDescription fieldDescription = ExcelEntityFieldDescription
                                .builder()
                                .excelColumn(annotation)
                                .field(field)
                                .reader(annotation.reader().newInstance())
                                .writer(annotation.writer().newInstance())
                                .build();
                        indexColumnMap.put(annotation.order(), fieldDescription);
                    } catch (InstantiationException e) {
                        e.printStackTrace();
                    }
                }
            }
        });
    }


}
