package com.chilunyc.fumao.common.util.excel.column.impl;

import com.chilunyc.fumao.common.util.CovertHelper;
import com.chilunyc.fumao.common.util.excel.ExcelEntityFieldDescription;
import com.chilunyc.fumao.common.util.excel.column.ExcelColumnReader;
import com.chilunyc.fumao.common.util.excel.column.ExcelColumnWriter;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 80347471@qq.com
 */
public class DefaultExcelColumnReaderWriter implements ExcelColumnReader, ExcelColumnWriter {

    /**
     * @param o
     * @param fieldDescription
     * @param row
     * @param rowIndex
     * @param cell
     * @param cellIndex
     */
    @Override
    public void read(Object o, ExcelEntityFieldDescription fieldDescription, Row row, int rowIndex, Cell cell, int cellIndex) throws Exception {
        String cellValue = null;
        Field field = fieldDescription.getField();
        Class<?> type = field.getType();
        switch (cell.getCellTypeEnum()) {
            case NUMERIC:
                // 是时间类型
                if (HSSFDateUtil.isCellDateFormatted(cell)) {
                    if (type.isAssignableFrom(Date.class)) {
                        field.set(o, cell.getDateCellValue());
                    } else {
                        throw new UnsupportedOperationException(String.format("cell[%s] 是时间类型, class[%s] field[%s] 不是时间类型", cell.getCellTypeEnum(), o.getClass(), field.getName()));
                    }
                }
                // 不是时间类型
                else {
                    cellValue = cell.getNumericCellValue() + "";
                    field.set(o, CovertHelper.stringTo(cellValue, type));
                }
                break;
            case STRING:
                cellValue = cell.getStringCellValue();
                field.set(o, CovertHelper.stringTo(cellValue, type));
                break;
            default:
                throw new UnsupportedOperationException(String.format("cell[%s] class[%s] field[%s]", cell.getCellTypeEnum(), o.getClass(), field.getName()));
        }
    }

    /**
     * @param o
     * @param value
     * @param row
     * @param i
     * @param cell
     * @param key
     * @param sheet
     * @throws Exception
     */
    @Override
    public void write(Object o, ExcelEntityFieldDescription value, XSSFRow row, int rowIndex, XSSFCell cell, int cellIndex, XSSFSheet sheet) throws Exception {
        Field field = value.getField();
        Object fieldValue = field.get(o);
        Class<?> type = field.getType();
        if (type.isAssignableFrom(String.class)) {
            cell.setCellValue((String) fieldValue);
        }
        //
        else if (type.isAssignableFrom(Integer.class) || type.isAssignableFrom(int.class)) {
            cell.setCellValue((int) fieldValue);
        }
        //
        else if (type.isAssignableFrom(Float.class) || type.isAssignableFrom(float.class)) {
            cell.setCellValue((float) fieldValue);
        }
        //
        else if (type.isAssignableFrom(Double.class) || type.isAssignableFrom(double.class)) {
            cell.setCellValue((Double) fieldValue);
        }
        //
        else if (type.isAssignableFrom(BigDecimal.class)) {
            cell.setCellValue(((BigDecimal) fieldValue).toString());
        }
        //
        else if (type.isAssignableFrom(Date.class)) {
            cell.setCellValue(DateFormatUtils.format(((Date) fieldValue), "yyyy-MM-dd HH:mm:ss"));
        }
        //
        else {
            throw new UnsupportedOperationException(String.format("src[%s] => class[%s]", fieldValue, type));
        }
    }
}
