package com.chilunyc.fumao.system.service;

import com.chilunyc.fumao.common.context.RequestHolder;
import com.chilunyc.fumao.common.exception.SystemException;
import com.chilunyc.fumao.common.log.model.EventTarget;
import com.chilunyc.fumao.common.log.model.EventType;
import com.chilunyc.fumao.common.log.service.EventService;
import com.chilunyc.fumao.common.request.IdsRequest;
import com.chilunyc.fumao.common.request.LoginRequest;
import com.chilunyc.fumao.common.util.jpa.query.QueryBuilder;
import com.chilunyc.fumao.common.util.security.SaltHelper;
import com.chilunyc.fumao.system.model.SystemAdmin;
import com.chilunyc.fumao.system.repository.SystemAdminRepository;
import com.chilunyc.fumao.system.request.systemAdmin.SystemAdminEditRequest;
import com.chilunyc.fumao.system.request.systemAdmin.SystemAdminPageRequest;
import lombok.extern.log4j.Log4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 */
@Log4j
@Service
@Transactional
public class SystemAdminService implements InitializingBean {


    @Autowired
    private SystemAdminRepository systemAdminRepository;

    @Autowired
    private EventService eventService;

    public Page<SystemAdmin> page(SystemAdminPageRequest request) {
        return systemAdminRepository.findAll(QueryBuilder.build(request), request.getPage());
    }


    public SystemAdmin find(Long id) {
        SystemAdmin one = systemAdminRepository.findOne(id);
        return one;
    }


    public SystemAdmin edit(SystemAdminEditRequest request) {
        SystemAdmin systemAdmin = new SystemAdmin();
        systemAdmin = systemAdminRepository.save(systemAdmin);
        eventService.add(RequestHolder.get(), EventTarget.System, EventType.编辑系统管理员, systemAdmin.getId(), request);
        return systemAdmin;
    }

    public void remove(IdsRequest<Long> request) {
        request
                .getIds()
                .forEach((id) -> {
                    systemAdminRepository.delete(id);
                });
        eventService.add(RequestHolder.get(), EventTarget.System, EventType.删除系统管理员, null, request.getIds());
    }


    @Override
    public void afterPropertiesSet() throws Exception {
        if (systemAdminRepository.existsByMobile("13888888888")) {
            return;
        }
        String salt = SaltHelper.salt();
        systemAdminRepository.save(
                SystemAdmin
                        .builder()
                        .mobile("13888888888")
                        .name("Tkk")
                        .salt(salt)
                        .password(SaltHelper.exec("!admin123", salt))
                        .build()
        );
    }

    public SystemAdmin login(LoginRequest request) {
        SystemAdmin systemAdmin = systemAdminRepository.findByMobile(request.getMobile());
        if (systemAdmin == null) {
            throw new SystemException("000004");
        } else if (SaltHelper.isEquals(systemAdmin.getPassword(), request.getPassword(), systemAdmin.getSalt())) {
            return systemAdmin;
        } else {
            throw new SystemException("000004");
        }
    }
}
